from leeger.model.league import YearSettings


def runAllChecks(yearSettings: YearSettings) -> None:
    """
    Runs all checks on the given YearSettings.
    The order in which these are called matters.
    """
    pass
